# Generated by Django 2.2.3 on 2019-08-20 11:24

from django.conf import settings
from django.db import migrations, models
import django_ilmoitin.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("django_ilmoitin", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="notificationtemplate",
            name="admin_notification_subject",
            field=models.CharField(
                blank=True,
                help_text="Subject for admins' notification",
                max_length=200,
                verbose_name="admin notification subject",
            ),
        ),
        migrations.AddField(
            model_name="notificationtemplate",
            name="admin_notification_text",
            field=models.TextField(
                blank=True,
                help_text="Text body for admins' notification.",
                verbose_name="admin notification text",
            ),
        ),
        migrations.AddField(
            model_name="notificationtemplate",
            name="admins_to_notify",
            field=models.ManyToManyField(
                blank=True,
                help_text="Choose admin users you want to be notified when this event happens.",
                related_name="_notificationtemplate_admins_to_notify_+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Admins to notify",
            ),
        ),
        migrations.AddField(
            model_name="notificationtemplate",
            name="from_email",
            field=models.EmailField(max_length=100, verbose_name="From email"),
        ),
    ]



from django import forms
from django.contrib.admin import site as admin_site
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _
from jinja2 import DebugUndefined, TemplateError
from jinja2.sandbox import SandboxedEnvironment
from parler.admin import TranslatableAdmin
from parler.forms import TranslatableModelForm

from .dummy_context import dummy_context
from .models import NotificationTemplate
from .registry import notifications


class NotificationTemplateForm(TranslatableModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Do not allow the admin to choose any of the template types that already
        # exist.
        existing_templates = NotificationTemplate.objects.all()
        if self.instance and self.instance.type:
            existing_templates = existing_templates.exclude(id=self.instance.id)
        used_types = set(existing_templates.values_list("type", flat=True))
        choices = [x for x in notifications.registry.items() if x[0] not in used_types]
        self.fields["type"] = forms.ChoiceField(choices=choices)

        admins_qs = (
            get_user_model()
            .objects.exclude(email="")
            .filter(Q(is_superuser=True) | Q(is_staff=True))
        )
        self.fields["admins_to_notify"].choices = [(a.id, a.email) for a in admins_qs]


class NotificationTemplateAdmin(TranslatableAdmin):
    form = NotificationTemplateForm
    change_form_template = "admin/preview_template.html"
    fieldsets = [
        (None, {"fields": ["type"]}),
        (_("User notification"), {"fields": ["subject", "body_html", "body_text"]}),
        (
            _("Admin notification"),
            {
                "fields": [
                    "admins_to_notify",
                    "admin_notification_subject",
                    "admin_notification_text",
                ]
            },
        ),
    ]

    def save_model(self, request, obj, form, change):
        if "_preview" not in request.POST:
            super(NotificationTemplateAdmin, self).save_model(
                request, obj, form, change
            )

    def response_change(self, request, obj):
        if "_preview" in request.POST:
            return self.preview(request, obj)
        else:
            return super(NotificationTemplateAdmin, self).response_change(request, obj)

    def response_add(self, request, obj, post_url_continue=None):
        if "_preview" in request.POST:
            return self.preview(request, obj)
        else:
            return super(NotificationTemplateAdmin, self).response_add(
                request, obj, post_url_continue=None
            )

    def preview(self, request, obj, **kwargs):
        env = SandboxedEnvironment(
            trim_blocks=True, lstrip_blocks=True, undefined=DebugUndefined
        )
        try:
            body_html = env.from_string(obj.body_html).render(
                dummy_context.get(obj.type)
            )
            return HttpResponse(body_html)
        except TemplateError as e:
            return HttpResponse(e)


admin_site.register(NotificationTemplate, NotificationTemplateAdmin)



import logging

from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import ugettext_lazy as _
from parler.models import TranslatableModel, TranslatedFields

from .registry import notifications

logger = logging.getLogger(__name__)


class NotificationTemplateException(Exception):
    pass


class NotificationTemplate(TranslatableModel):
    type = models.CharField(max_length=50, verbose_name=_("type"), unique=True)

    admins_to_notify = models.ManyToManyField(
        get_user_model(),
        related_name="+",
        blank=True,
        verbose_name=_("Admins to notify"),
        help_text=_(
            "Choose admin users you want to be notified when this event happens."
        ),
    )
    admin_notification_subject = models.CharField(
        verbose_name=_("admin notification subject"),
        max_length=200,
        help_text=_("Subject for admins' notification"),
        blank=True,
    )
    admin_notification_text = models.TextField(
        verbose_name=_("admin notification text"),
        help_text=_("Text body for admins' notification."),
        blank=True,
    )

    translations = TranslatedFields(
        subject=models.CharField(verbose_name=_("subject"), max_length=255),
        body_html=models.TextField(verbose_name=_("body, HTML version"), blank=True),
        body_text=models.TextField(
            verbose_name=_("body, plain text version"),
            blank=True,
            help_text=_(
                "If left blank, the HTML version without HTML tags will be used."
            ),
        ),
    )

    class Meta:
        verbose_name = _("notification")
        verbose_name_plural = _("notifications")

    def __str__(self):
        if self.type in notifications.registry:
            return str(notifications.registry[self.type])
        else:
            return "{} ({})".format(self.type, _("disabled"))



from collections import defaultdict
from copy import deepcopy
from enum import Enum
from typing import Any, Mapping, Type, Union

from .registry import notifications

_StrOrEnum = Union[str, Type[Enum]]

COMMON_CONTEXT = "__common__"


class DummyContext:
    context = defaultdict(dict)

    def update(self, context_dict: Mapping[_StrOrEnum, Mapping[str, Any]]) -> None:
        """
        Proxy helper method that makes updating
        DummyContext's context storage less verbose.

        :param context_dict: dictionary of values to be put into dummy context
        """
        assert isinstance(
            context_dict, Mapping
        ), "DummyContext's update() method accepts only dictionaries."

        for type_code, context in context_dict.items():
            if isinstance(type_code, Enum):
                type_code = type_code.value

            self._assert_type_code_is_str(type_code)
            if not type_code == COMMON_CONTEXT:
                self._assert_notification_type_exists(type_code)

            self.context.update({type_code: context})

    def get(self, type_code: _StrOrEnum) -> dict:
        """
        Proxy helper method that makes fetching from
        DummyContext's context storage less verbose.

        :param type_code: code of the notification type
        :return: dummy context for the notification type
        """
        if isinstance(type_code, Enum):
            type_code = type_code.value

        self._assert_type_code_is_str(type_code)
        self._assert_notification_type_exists(type_code)

        context = deepcopy(self.context.get(COMMON_CONTEXT)) or {}
        context.update(self.context.get(type_code, {}))
        return context

    @staticmethod
    def _assert_type_code_is_str(type_code: str) -> None:
        assert isinstance(type_code, str), (
            "Expected a string code for a notification"
            "template's type, received {}".format(type(type_code))
        )

    @staticmethod
    def _assert_notification_type_exists(type_code: str) -> None:
        assert type_code in notifications.registry, (
            "This type of notification is not"
            "registered with the ilmoitin: {}".format(type_code)
        )


dummy_context = DummyContext()



# Generated by Django 2.2.8 on 2020-01-03 21:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("django_ilmoitin", "0002_add_admin_notifications")]

    operations = [
        migrations.RemoveField(model_name="notificationtemplate", name="from_email")
    ]



import os

from setuptools import find_packages, setup

from django_ilmoitin import __version__

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django_ilmoitin",
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    license="MIT License",
    description="Django app for sending notifications.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/City-of-Helsinki/django-ilmoitin",
    author="City of Helsinki",
    author_email="dev@hel.fi",
    install_requires=[
        "Django<3.0",
        "django-parler<2.0",
        "django-anymail",
        "django-mailer",
        "jinja2",
    ],
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "pytest-django"],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)
